@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+# -*- coding: utf-8 -*-
 from __future__ import print_function
 
 COPYRIGHT = """\
@@ -25,30 +25,19 @@ import json
 import os
 import re
 import shlex
+import six
 import subprocess
 import sys
 import textwrap
 
 import pkg_resources
 import requests
+from six.moves import configparser
+from six.moves import input as do_input
+from six.moves.urllib.parse import urlencode
+from six.moves.urllib.parse import urljoin
+from six.moves.urllib.parse import urlparse
 
-if sys.version < '3':
-    import ConfigParser
-    import urllib
-    import urlparse
-    urlencode = urllib.urlencode
-    urljoin = urlparse.urljoin
-    urlparse = urlparse.urlparse
-    do_input = raw_input
-else:
-    import configparser as ConfigParser
-
-    import urllib.parse
-    import urllib.request
-    urlencode = urllib.parse.urlencode
-    urljoin = urllib.parse.urljoin
-    urlparse = urllib.parse.urlparse
-    do_input = input
 
 VERBOSE = False
 UPDATE = False
@@ -76,7 +65,7 @@ class colors(object):
 
 
 class GitReviewException(Exception):
-    EXIT_CODE = 127
+    EXIT_CODE = 1
 
 
 class CommandFailed(GitReviewException):
@@ -134,7 +123,7 @@ def run_command_status(*argv, **kwargs):
         print(datetime.datetime.now(), "Running:", " ".join(argv))
     if len(argv) == 1:
         # for python2 compatibility with shlex
-        if sys.version_info < (3,) and isinstance(argv[0], unicode):
+        if sys.version_info < (3,) and isinstance(argv[0], six.text_type):
             argv = shlex.split(argv[0].encode('utf-8'))
         else:
             argv = shlex.split(str(argv[0]))
@@ -163,7 +152,7 @@ def run_command_exc(klazz, *argv, **env):
     klazz should be derived from CommandFailed
     """
     (rc, output) = run_command_status(*argv, **env)
-    if rc != 0:
+    if rc:
         raise klazz(rc, output, argv, env)
     return output
 
@@ -254,7 +243,7 @@ def run_custom_script(action):
             returns.append((status, output, fpath))
 
     for (status, output, path) in returns:
-        if status is not None and status != 0:
+        if status:
             raise CustomScriptException(status, output, [path], {})
         elif output and VERBOSE:
             print("script %s output is:" % (path))
@@ -272,12 +261,15 @@ def git_config_get_value(section, option, default=None, as_bool=False):
     try:
         result = run_command_exc(GitConfigException, *cmd).strip()
         if VERBOSE:
-            print(datetime.datetime.now(), "result:", result)
+            print(datetime.datetime.now(), "... %s.%s = %s"
+                  % (section, option, result))
         return result
     except GitConfigException as exc:
         if exc.rc == 1:
-            if VERBOSE:
-                print(datetime.datetime.now(), "using default:", default)
+            if VERBOSE and default is not None:
+                print(datetime.datetime.now(),
+                      "... nothing in git config, returning func parameter:",
+                      default)
             return default
         raise
 
@@ -300,9 +292,16 @@ class Config(object):
                 self.config.update(load_config_file(filename))
 
     def __getitem__(self, key):
+        """Let 'git config --get' override every Config['key'] access"""
         value = git_config_get_value('gitreview', key)
         if value is None:
             value = self.config[key]
+        # "--verbose" doesn't trace *early* invocations; for that you
+        # must change the value at the top of this file (*and* pass
+        # --verbose)
+        if VERBOSE:
+            print(datetime.datetime.now(),
+                  "Config['%s'] = %s " % (key, value))
         return value
 
 
@@ -338,10 +337,10 @@ def set_hooks_commit_msg(remote, target_file):
         else:
             (hostname, username, port, project_name) = \
                 parse_gerrit_ssh_params_from_git_url(remote_url)
-            if username is None:
-                userhost = hostname
-            else:
+            if username:
                 userhost = "%s@%s" % (username, hostname)
+            else:
+                userhost = hostname
             # OS independent target file
             scp_target_file = target_file.replace(os.sep, "/")
             cmd = ["scp", userhost + ":hooks/commit-msg", scp_target_file]
@@ -376,10 +375,10 @@ def make_remote_url(scheme, username, hostname, port, project):
     if port is None and scheme == 'ssh':
         port = 29418
     hostport = '%s:%s' % (hostname, port) if port else hostname
-    if username is None:
-        return "%s://%s/%s" % (scheme, hostport, project)
-    else:
+    if username:
         return "%s://%s@%s/%s" % (scheme, username, hostport, project)
+    else:
+        return "%s://%s/%s" % (scheme, hostport, project)
 
 
 def add_remote(scheme, hostname, port, project, remote, usepushurl):
@@ -413,7 +412,7 @@ def add_remote(scheme, hostname, port, project, remote, usepushurl):
     print("\t%s" % remote_url)
 
     (status, remote_output) = run_command_status(cmd)
-    if status != 0:
+    if status:
         raise CommandFailed(status, remote_output, cmd, {})
 
     if asked_for_username:
@@ -526,7 +525,7 @@ def parse_gerrit_ssh_params_from_git_url(git_url):
             hostname = parsed_url.path[2:].split("/")[0]
 
         if "@" in hostname:
-            (username, hostname) = hostname.split("@")
+            (username, _, hostname) = hostname.rpartition("@")
         if ":" in hostname:
             (hostname, port) = hostname.split(":")
 
@@ -704,7 +703,7 @@ def check_color_support():
 
 def load_config_file(config_file):
     """Load configuration options from a file."""
-    configParser = ConfigParser.ConfigParser()
+    configParser = configparser.ConfigParser()
     configParser.read(config_file)
     options = {
         'scheme': 'scheme',
@@ -806,8 +805,8 @@ def check_remote(branch, remote, scheme, hostname, port, project,
             printwrap("Please set the push-url on your origin remote to the "
                       "location of your gerrit server and try again")
         else:
-            printwrap("Please manually create a remote "
-                      "named \"%s\" and try again." % remote)
+            printwrap("Please manually create a remote named \"%s\" or "
+                      "rename the default one and try again." % remote)
         sys.exit(1)
 
     # Gerrit remote not present, try to add it
@@ -939,61 +938,25 @@ def assert_one_change(remote, branch, yes, have_hook):
                 sys.exit(1)
 
 
-def use_topic(why, topic):
-    """Inform the user about why a particular topic has been selected."""
-    if VERBOSE:
-        print(why % ('"%s"' % topic,))
-    return topic
-
-
 def get_topic(target_branch):
-
     branch_name = get_branch_name(target_branch)
 
     branch_parts = branch_name.split("/")
     if len(branch_parts) >= 3 and branch_parts[0] == "review":
-        return use_topic("Using change number %s "
-                         "for the topic of the change submitted",
-                         "/".join(branch_parts[2:]))
+        # We don't want to set the review number as the topic
+        if branch_parts[2].isdigit():
+            return
 
-    preferred_log_format = "%B"
-    log_output = run_command("git log --pretty='" + preferred_log_format +
-                             "' HEAD^1..HEAD")
-    if log_output == preferred_log_format:
-        # The %B format specifier is supported starting at Git v1.7.2. If it's
-        # not supported, we'll just get back '%B', so we try something else.
-        # The downside of %s is that it removes newlines in the subject.
-        log_output = run_command("git log --pretty='%s%n%b' HEAD^1..HEAD")
-    bug_re = r'''(?x)                # verbose regexp
-                 \b([Bb]ug|[Ll][Pp]) # bug or lp
-                 [ \t\f\v]*          # don't want to match newline
-                 [:]?                # separator if needed
-                 [ \t\f\v]*          # don't want to match newline
-                 [#]?                # if needed
-                 [ \t\f\v]*          # don't want to match newline
-                 (\d+)               # bug number'''
+        topic = "/".join(branch_parts[2:])
+        if VERBOSE:
+            print("Using change number %s for the topic of the change "
+                  "submitted" % topic)
+        return topic
 
-    match = re.search(bug_re, log_output)
-    if match is not None:
-        return use_topic("Using bug number %s "
-                         "for the topic of the change submitted",
-                         "bug/%s" % match.group(2))
-
-    bp_re = r'''(?x)                         # verbose regexp
-                \b([Bb]lue[Pp]rint|[Bb][Pp]) # a blueprint or bp
-                [ \t\f\v]*                   # don't want to match newline
-                [#:]?                        # separator if needed
-                [ \t\f\v]*                   # don't want to match newline
-                ([0-9a-zA-Z-_]+)             # any identifier or number'''
-    match = re.search(bp_re, log_output)
-    if match is not None:
-        return use_topic("Using blueprint number %s "
-                         "for the topic of the change submitted",
-                         "bp/%s" % match.group(2))
-
-    return use_topic("Using local branch name %s "
-                     "for the topic of the change submitted",
-                     branch_name)
+    if VERBOSE:
+        print("Using local branch name %s for the topic of the change "
+              "submitted" % branch_name)
+    return branch_name
 
 
 class CannotQueryOpenChangesets(CommandFailed):
@@ -1232,7 +1195,7 @@ def fetch_review(review, masterbranch, remote, project):
 
     print("Downloading %s from gerrit" % refspec)
     run_command_exc(PatchSetGitFetchFailed,
-                    "git", "fetch", remote, refspec)
+                    "git", "fetch", remote_url, refspec)
     return branch_name, remote_branch
 
 
@@ -1737,11 +1700,21 @@ def _main():
 
 
 def main():
+    # workaround for avoiding UnicodeEncodeError on print() with older python
+    if sys.version_info[0] < 3:
+        # without reload print would fail even if sys.stdin.encoding
+        # would report utf-8
+        # see: https://stackoverflow.com/a/23847316/99834
+        stdin, stdout, stderr = sys.stdin, sys.stdout, sys.stderr
+        reload(sys)
+        sys.stdin, sys.stdout, sys.stderr = stdin, stdout, stderr
+        sys.setdefaultencoding(os.environ.get('PYTHONIOENCODING', 'utf-8'))
+
     try:
         _main()
     except GitReviewException as e:
         print(e)
-        sys.exit(getattr(e, 'EXIT_CODE', -1))
+        sys.exit(e.EXIT_CODE)
 
 
 if __name__ == "__main__":
