@@ -18,6 +18,7 @@
 import json
 import os
 import shutil
+import testtools
 
 from git_review import tests
 from git_review.tests import utils
@@ -286,9 +287,10 @@ class GitReviewTestCase(tests.BaseGitReviewTestCase):
 
         br_out = self._run_git('checkout',
                                '-b', 'test_branch', 'origin/maint')
-        expected_track = 'Branch test_branch set up to track remote' + \
-                         ' branch maint from origin.'
-        self.assertIn(expected_track, br_out)
+        expected_track = ".*\nBranch '?test_branch'? set up to track remote" + \
+                         " branch '?maint'? from '?origin'?."
+        track_matcher = testtools.matchers.MatchesRegex(expected_track)
+        self.assertThat(br_out, track_matcher)
         branches = self._run_git('branch', '-a')
         expected_branch = '* test_branch'
         observed = branches.split('\n')
@@ -383,98 +385,27 @@ class GitReviewTestCase(tests.BaseGitReviewTestCase):
         finally:
             os.environ.update(LANG=lang_env)
 
+    def test_no_topic(self):
+        """Test on change with no topic.
+
+        This will be checked out as 'review/{owner}/{ID}'. We don't want to set
+        the topic to '{ID}'.
+        """
+        self._run_git_review('-s')
+        curr_branch = self._run_git('rev-parse', '--abbrev-ref', 'HEAD')
+        self._run_git('checkout', '-b', 'review/johndoe/123456')
+        self._simple_change('test file modified', 'derp derp derp')
+        self._assert_branch_would_be(curr_branch)
+
     def test_git_review_t(self):
         self._run_git_review('-s')
         self._simple_change('test file modified', 'commit message for bug 654')
         self._assert_branch_would_be('master%topic=zat',
                                      extra_args=['-t', 'zat'])
 
-    def test_bug_topic(self):
-        self._run_git_review('-s')
-        self._simple_change('a change', 'new change for bug 123')
-        self._assert_branch_would_be('master%topic=bug/123')
-
-    def test_bug_topic_newline(self):
-        self._run_git_review('-s')
-        self._simple_change('a change', 'new change not for bug\n\n123')
-        self._assert_branch_would_be('master')
-
-    def test_bp_topic(self):
-        self._run_git_review('-s')
-        self._simple_change('a change', 'new change for blueprint asdf')
-        self._assert_branch_would_be('master%topic=bp/asdf')
-
-    def test_bp_topic_newline(self):
-        self._run_git_review('-s')
-        self._simple_change('a change', 'new change not for blueprint\n\nasdf')
-        self._assert_branch_would_be('master')
-
-    def test_author_name_topic_bp(self):
-        old_author = None
-        if 'GIT_AUTHOR_NAME' in os.environ:
-            old_author = os.environ['GIT_AUTHOR_NAME']
-        try:
-            os.environ['GIT_AUTHOR_NAME'] = 'BPNAME'
-            self._run_git_review('-s')
-            self._simple_change('a change',
-                                'new change 1 with name but no topic')
-            self._assert_branch_would_be('master')
-        finally:
-            if old_author:
-                os.environ['GIT_AUTHOR_NAME'] = old_author
-            else:
-                del os.environ['GIT_AUTHOR_NAME']
-
-    def test_author_email_topic_bp(self):
-        old_author = None
-        if 'GIT_AUTHOR_EMAIL' in os.environ:
-            old_author = os.environ['GIT_AUTHOR_EMAIL']
-        try:
-            os.environ['GIT_AUTHOR_EMAIL'] = 'bpemail@example.com'
-            self._run_git_review('-s')
-            self._simple_change('a change',
-                                'new change 1 with email but no topic')
-            self._assert_branch_would_be('master')
-        finally:
-            if old_author:
-                os.environ['GIT_AUTHOR_EMAIL'] = old_author
-            else:
-                del os.environ['GIT_AUTHOR_EMAIL']
-
-    def test_author_name_topic_bug(self):
-        old_author = None
-        if 'GIT_AUTHOR_NAME' in os.environ:
-            old_author = os.environ['GIT_AUTHOR_NAME']
-        try:
-            os.environ['GIT_AUTHOR_NAME'] = 'Bug: #1234'
-            self._run_git_review('-s')
-            self._simple_change('a change',
-                                'new change 2 with name but no topic')
-            self._assert_branch_would_be('master')
-        finally:
-            if old_author:
-                os.environ['GIT_AUTHOR_NAME'] = old_author
-            else:
-                del os.environ['GIT_AUTHOR_NAME']
-
-    def test_author_email_topic_bug(self):
-        old_author = None
-        if 'GIT_AUTHOR_EMAIL' in os.environ:
-            old_author = os.environ['GIT_AUTHOR_EMAIL']
-        try:
-            os.environ['GIT_AUTHOR_EMAIL'] = 'bug5678@example.com'
-            self._run_git_review('-s')
-            self._simple_change('a change',
-                                'new change 2 with email but no topic')
-            self._assert_branch_would_be('master')
-        finally:
-            if old_author:
-                os.environ['GIT_AUTHOR_EMAIL'] = old_author
-            else:
-                del os.environ['GIT_AUTHOR_EMAIL']
-
     def test_git_review_T(self):
         self._run_git_review('-s')
+        self._run_git('checkout', '-b', 'bug/456')
         self._simple_change('test file modified', 'commit message for bug 456')
         self._assert_branch_would_be('master%topic=bug/456')
         self._assert_branch_would_be('master', extra_args=['-T'])
